
##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for advice

This module was adapted from 'protocols.tests.advice', part of the Python
Enterprise Application Kit (PEAK).  Please notify the PEAK authors
(pje@telecommunity.com and tsarna@sarna.org) if bugs are found or
Zope-specific changes are required, so that the PEAK version of this module
can be kept in sync.

PEAK is a Python application framework that interoperates with (but does
not require) Zope 3 and Twisted.  It provides tools for manipulating UML
models, object-relational persistence, aspect-oriented programming, and more.
Visit the PEAK home page at http://peak.telecommunity.com for more information.

$Id: test_advice.py 40836 2005-12-16 22:40:51Z benji_york $
"""

import sys
import unittest
from repoze.component import advice

class FrameInfoTest(unittest.TestCase):

    classLevelFrameInfo = advice.getFrameInfo(sys._getframe())

    def testModuleInfo(self):
        kind, module, f_locals, f_globals = moduleLevelFrameInfo
        self.assertEqual(kind, "module")
        for d in module.__dict__, f_locals, f_globals:
            self.assertTrue(d is globals())

    def testClassicClassInfo(self):
        kind, module, f_locals, f_globals = ClassicClass.classLevelFrameInfo
        self.assertEqual(kind, "class")

        self.assertTrue(f_locals is ClassicClass.__dict__)  # ???
        for d in module.__dict__, f_globals:
            self.assertTrue(d is globals())

    def testNewStyleClassInfo(self):
        kind, module, f_locals, f_globals = NewStyleClass.classLevelFrameInfo
        self.assertEqual(kind, "class")

        for d in module.__dict__, f_globals:
            self.assertTrue(d is globals())

    def testCallInfo(self):
        kind, module, f_locals, f_globals = advice.getFrameInfo(sys._getframe())
        self.assertEqual(kind, "function call")
        self.assertTrue(f_locals is locals()) # ???
        for d in module.__dict__, f_globals:
            self.assertTrue(d is globals())


class AdviceTests(unittest.TestCase):

    def testOrder(self):
        log = []
        class Foo(object):
            ping(log, 1)
            ping(log, 2)
            ping(log, 3)

        # Strip the list nesting
        for i in 1,2,3:
            self.assertTrue(isinstance(Foo, list))
            Foo, = Foo

        self.assertEqual(log, [(1, Foo), (2, [Foo]), (3, [[Foo]])])

    def testOutside(self): # pragma: no cover
        if sys.version_info < (2, 6):
            try:
                ping([], 1)
            except SyntaxError:
                pass
            else:
                raise AssertionError(
                    "Should have detected advice outside class body"
                )

    def testDoubleType(self): # pragma: no cover
        if sys.hexversion >= 0x02030000:
            return  # you can't duplicate bases in 2.3
        class aType(type,type):
            ping([],1)
        aType, = aType
        self.assertTrue(aType.__class__ is type)

    def testSingleExplicitMeta(self):

        class M(type):
            pass

        class C(M, metaclass=M):
            ping([],1)

        C, = C
        self.assertTrue(C.__class__ is M)


    def testMixedMetas(self):

        class M1(type): pass
        class M2(type): pass

        class B1(metaclass=M1): pass
        class B2(metaclass=M2): pass

        try:
            class C(B1,B2):
                ping([],1)
        except TypeError:
            pass
        else: # pragma: no cover
            raise AssertionError("Should have gotten incompatibility error")

        class M3(M1,M2): pass

        class C(B1,B2, metaclass=M3):
            ping([],1)

        self.assertTrue(isinstance(C,list))
        C, = C
        self.assertTrue(isinstance(C,M3))

    def testMetaOfClass(self):

        class metameta(type):
            pass

        class meta(type, metaclass=metameta):
            pass

        self.assertEqual(advice.determineMetaclass((meta, type)), metameta)

from types import ClassType

def ping(log, value):

    def pong(klass):
        log.append((value,klass))
        return [klass]

    advice.addClassAdvisor(pong)

class ClassicClass(metaclass=ClassType):
    classLevelFrameInfo = advice.getFrameInfo(sys._getframe())

class NewStyleClass(metaclass=type):
    classLevelFrameInfo = advice.getFrameInfo(sys._getframe())

moduleLevelFrameInfo = advice.getFrameInfo(sys._getframe())

